package gregapi.oredict;

import gregapi.code.ArrayListNoNulls;
import gregapi.util.OM;

import java.util.*;

import net.minecraft.item.ItemStack;

/**
 * @author Gregorius Techneticies
 */
public class OreDictItemData {
	private static final OreDictMaterialStack[] EMPTY_MATERIALSTACK_ARRAY = new OreDictMaterialStack[0];

	public boolean mBlackListed = false;
	public boolean mBlocked = false;
	public boolean mUseVanillaDamage = false;
	public ItemStack mUnificationTarget = null;
	
	/** The OreDictPrefix if there is one assigned to this. */
	public final OreDictPrefix mPrefix;
	/** The OreDictMaterialStack containing the Main Material of this Item. The Amount is in Material Units (U) */
	public final OreDictMaterialStack mMaterial;
	/** The OreDictMaterialStack containing the remaining Byproduct Materials of this Item. The Amount is in Material Units (U). */
	public final OreDictMaterialStack[] mByProducts;
	
	public OreDictItemData(OreDictPrefix aPrefix, OreDictMaterial aMaterial) {
		mPrefix = aPrefix;
		mMaterial = aMaterial==null?null:OM.stack(aMaterial, aPrefix.mAmount);
		mByProducts = aPrefix.mByProducts.isEmpty()?EMPTY_MATERIALSTACK_ARRAY:aPrefix.mByProducts.toArray(new OreDictMaterialStack[aPrefix.mByProducts.size()]);
	}
	
	public OreDictItemData(OreDictMaterialStack aMaterial, OreDictMaterialStack... aByProducts) {
		mPrefix = null;
		mMaterial = aMaterial.mMaterial==null?null:aMaterial.clone();
		mBlackListed = true;
		if (aByProducts == null) {
			mByProducts = EMPTY_MATERIALSTACK_ARRAY;
		} else {
			OreDictMaterialStack[] tByProducts = aByProducts.length<1?EMPTY_MATERIALSTACK_ARRAY:new OreDictMaterialStack[aByProducts.length];
			int j = 0;
			for (int i = 0; i < aByProducts.length; i++) if (aByProducts[i] != null && aByProducts[i].mMaterial != null) tByProducts[j++] = aByProducts[i].clone();
			mByProducts = j>0?new OreDictMaterialStack[j]:EMPTY_MATERIALSTACK_ARRAY;
			for (int i = 0; i < mByProducts.length; i++) mByProducts[i] = tByProducts[i];
		}
	}
	
	public OreDictItemData(OreDictMaterial aMaterial, long aAmount, OreDictMaterialStack... aByProducts) {
		this(OM.stack(aMaterial, aAmount), aByProducts);
	}
	
	public OreDictItemData(OreDictMaterial aMaterial, long aAmount, OreDictMaterial aByProduct, long aByProductAmount) {
		this(OM.stack(aMaterial, aAmount), OM.stack(aByProduct, aByProductAmount));
	}
	
	public OreDictItemData(OreDictItemData... aData) {
		mPrefix = null;
		mBlackListed = true;
		
		ArrayList<OreDictMaterialStack> aList = new ArrayListNoNulls<OreDictMaterialStack>(), rList = new ArrayListNoNulls<OreDictMaterialStack>();
		
		for (OreDictItemData tData : aData) if (tData != null) {
			if (tData.hasValidMaterialData() && tData.mMaterial.mAmount > 0) aList.add(tData.mMaterial.clone());
			for (OreDictMaterialStack tMaterial : tData.mByProducts) if (tMaterial.mAmount > 0) aList.add(tMaterial.clone());
		}
		
		for (OreDictMaterialStack aMaterial : aList) {
			boolean temp = true;
			for (OreDictMaterialStack tMaterial : rList) if (aMaterial.mMaterial == tMaterial.mMaterial) {
				tMaterial.mAmount += aMaterial.mAmount;
				temp = false; break;
			}
			if (temp) rList.add(aMaterial.clone());
		}
		
		Collections.sort(rList, new Comparator<OreDictMaterialStack>() {@Override public int compare(OreDictMaterialStack a, OreDictMaterialStack b) {return a.mAmount == b.mAmount ? 0 : a.mAmount > b.mAmount ? -1 : +1;}});
		
		if (rList.isEmpty()) {
			mMaterial = null;
		} else {
			mMaterial = rList.get(0);
			rList.remove(0);
		}
		
		mByProducts = rList.toArray(new OreDictMaterialStack[rList.size()]);
	}
	
	public boolean hasValidPrefixMaterialData() {
		return mPrefix != null && mMaterial != null && mMaterial.mMaterial != null;
	}
	
	public boolean hasValidPrefixData() {
		return mPrefix != null;
	}
	
	public boolean hasValidMaterialData() {
		return mMaterial != null && mMaterial.mMaterial != null;
	}
	
	/** Utility Function for getting a List containing both, the Main Material and all the Byproduct Materials. The Amount is in Material Units (U). */
	public List<OreDictMaterialStack> getAllMaterialStacks() {
		ArrayListNoNulls<OreDictMaterialStack> rList = new ArrayListNoNulls(mByProducts.length + 1);
		if (hasValidMaterialData()) rList.add(mMaterial);
		rList.addAll(Arrays.asList(mByProducts));
		return rList;
	}
	
	/** Utility Function for getting a Byproduct Material at a certain Index, if it exists. The Amount is in Material Units (U). */
	public OreDictMaterialStack getByProduct(int aIndex) {
		return aIndex>=0&&aIndex<mByProducts.length?mByProducts[aIndex]:null;
	}
	
	@Override
	public String toString() {
		if (mPrefix == null || mMaterial == null || mMaterial.mMaterial == null) return "";
		return mPrefix.mNameInternal + mMaterial.mMaterial.mNameInternal;
	}
	
	public OreDictItemData setUseVanillaDamage() {
		mUseVanillaDamage = true;
		return this;
	}
	
	public static OreDictItemData copy(OreDictItemData aData) {
		return aData == null ? null : aData.copy();
	}
	
	public OreDictItemData copy() {
		OreDictItemData rData = mPrefix == null ? new OreDictItemData(mMaterial, mByProducts) : new OreDictItemData(mPrefix, mMaterial.mMaterial);
		rData.mUnificationTarget = mUnificationTarget;
		rData.mUseVanillaDamage = mUseVanillaDamage;
		rData.mBlackListed = mBlackListed;
		rData.mBlocked = mBlocked;
		return rData;
	}
}